
/* readplugs.cpp
 * -------------
 * Purpose: Create a list of plugins used in a an IT/MPTM file.
 * Author: Johannes Schultz
 * License: New BSD
 *
 */

#include <iostream>
#include <fstream>
#include "readplugs.hpp"

using namespace std;

template<typename T>
T readInt(fstream &fIn)
{
	T n;
	fIn.read((char *)&n, sizeof(T));
	return n;
}

int main(int argc, char *argv[])
{

	if(argc != 2)
	{
		cout << "Parameter needed: Filename In" << endl;
		return 1;
	}

	fstream fIn;
	fIn.open(argv[1], ios::in | ios::binary);

	ITFileHeader fileHeader;
	fIn.read((char *)&fileHeader, sizeof(fileHeader));
	if(memcmp(&fileHeader.id, "IMPM", 4))
	{
		cout << "Not a valid IT/MPTM file!" << endl;
		return 1;
	}

	// Skip pointers
	fIn.seekg(fileHeader.ordnum + (fileHeader.insnum + fileHeader.smpnum + fileHeader.patnum) * 4, ios::cur);

	// Skip edit history
	if(fileHeader.special & ITFileHeader::embedEditHistory)
	{
		uint16_t nflt = readInt<uint16_t>(fIn);
		fIn.seekg(nflt * 8, ios::cur);
	}

	// Skip MIDI config
	if((fileHeader.flags & ITFileHeader::reqEmbeddedMIDIConfig) || (fileHeader.special & ITFileHeader::embedMIDIConfiguration))
	{
		fIn.seekg(4896, ios::cur);
	}

	while(!fIn.eof())
	{
		char code[4];
		fIn.read(code, 4);
		uint32_t size = readInt<uint32_t>(fIn);

		if(!memcmp(code, "XTPM", 4))
		{
			// End
			break;
		} else if(memcmp(code, "FX00", 4) >= 0 && memcmp(code, "FX99", 4) <= 0)
		{
			PluginInfo plugInfo;
			fIn.read((char *)&plugInfo, sizeof(plugInfo));
			fIn.seekg(size - sizeof(plugInfo), ios::cur);

			plugInfo.szName[31] = 0;
			plugInfo.szLibraryName[63] = 0;
			cout << code[2] << code[3] << ": " << plugInfo.szLibraryName << /*"\t" << plugInfo.szName <<*/ endl;
		} else
		{
			// Unknown code
			fIn.seekg(size, ios::cur);
		}
	}

	fIn.close();

	return 0;
}