
/* readplugs.hpp
 * -------------
 * Purpose: Header file for readplugs.cpp
 * Author: Johannes Schultz
 * License: New BSD
 *
 */


///////////////
// Data types
#ifdef _MSC_VER
typedef unsigned __int8 uint8_t;
typedef unsigned __int16 uint16_t;
typedef unsigned __int32 uint32_t;
#else
#include <stdbool.h>
#endif


#pragma pack(push, 1)

struct ITFileHeader
{
	// Magic Bytes
	enum Magic
	{
		itMagic = 0x4D504D49,		// "IMPM" IT Header Magic Bytes
		mptmMagic = 0x2E6D7074,		// "tpm." Old MPTM header magic bytes
		omptMagic = 0x54504D4F,		// "OMPT" Magic Bytes for non-standard OpenMPT IT files
		chibiMagic = 0x49424843,	// "CHBI" Magic Bytes in the IT header to identify ChibiTracker
	};

	// Header Flags
	enum ITHeaderFlags
	{
		useStereoPlayback		= 0x01,
		vol0Optimisations		= 0x02,
		instrumentMode			= 0x04,
		linearSlides			= 0x08,
		itOldEffects			= 0x10,
		itCompatGxx				= 0x20,
		useMIDIPitchController	= 0x40,
		reqEmbeddedMIDIConfig	= 0x80,
		extendedFilterRange		= 0x1000,
	};

	// Special Flags
	enum ITHeaderSpecialFlags
	{
		embedSongMessage		= 0x01,
		embedEditHistory		= 0x02,
		embedPatternHighlights	= 0x04,
		embedMIDIConfiguration	= 0x08,
	};

	uint32_t id;				// Magic Bytes (IMPM)
	char     songname[26];		// Song Name, null-terminated (but may also contain nulls)
	uint8_t  highlight_minor;	// Rows per Beat highlight
	uint8_t  highlight_major;	// Rows per Measure highlight
	uint16_t ordnum;			// Number of Orders
	uint16_t insnum;			// Number of Instruments
	uint16_t smpnum;			// Number of Samples
	uint16_t patnum;			// Number of Patterns
	uint16_t cwtv;				// "Made With" Tracker
	uint16_t cmwt;				// "Compatible With" Tracker
	uint16_t flags;				// Header Flags
	uint16_t special;			// Special Flags, for embedding extra information
	uint8_t  globalvol;			// Global Volume (0...128)
	uint8_t  mv;				// Master Volume (0...128), referred to as Sample Volume in OpenMPT
	uint8_t  speed;				// Initial Speed (1...255)
	uint8_t  tempo;				// Initial Tempo (31...255)
	uint8_t  sep;				// Pan Separation (0...128)
	uint8_t  pwd;				// Pitch Wheel Depth
	uint16_t msglength;			// Length of Song Message
	uint32_t msgoffset;			// Offset of Song Message in File (IT crops message after first null)
	uint32_t reserved;			// ChibiTracker writes "CHBI" here. OpenMPT writes "OMPT" here in some cases, see Load_it.cpp
	uint8_t  chnpan[64];		// Initial Channel Panning
	uint8_t  chnvol[64];		// Initial Channel Volume
};

struct PluginInfo
{
	// dwInputRouting flags
	enum RoutingFlags
	{
		irApplyToMaster	= 0x01,		// Apply to master mix
		irBypass		= 0x02,		// Bypass effect
		irWetMix		= 0x04,		// Wet Mix (dry added)
		irExpandMix		= 0x08,		// [0%,100%] -> [-200%,200%]
	};

	uint32_t dwPluginId1;			// Plugin type (kEffectMagic, kDmoMagic, kBuzzMagic)
	uint32_t dwPluginId2;			// Plugin unique ID
	uint8_t routingFlags;			// See RoutingFlags
	uint8_t mixMode;
	uint8_t gain;					// Divide by 10 to get real gain
	uint8_t reserved;
	uint32_t dwOutputRouting;		// 0 = send to master 0x80 + x = send to plugin x
	uint32_t dwReserved[4];			// Reserved for routing info
	char szName[32];				// User-chosen plugin name
	char szLibraryName[64];			// original DLL name
};

#pragma pack(pop)
